// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.key;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.editor.FormEditor;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyGraphEditor;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyListEditor;
import eu.etaxonomy.taxeditor.model.DataChangeBridge;
import eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;

/**
 * @author n.hoffmann
 * @created Mar 30, 2011
 * @version 1.0
 */
public class KeyEditor extends FormEditor implements IConversationEnabled,
		IDirtyMarkableSelectionProvider, IPartContentHasDetails {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.key";

	private ConversationHolder conversation;

	private boolean dirty;

	private KeyEditorDataChangeBehaviour dataChangeBehavior;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor monitor) {
		try {
			monitor.beginTask("Saving Editor", 1);
			getConversationHolder().bind();
			getConversationHolder().commit(true);
			setDirty(false);
			monitor.worked(1);
		} finally {
			monitor.done();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	@Override
	public void doSaveAs() {
		// not allowed
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void setFocus() {
		getConversationHolder().bind();
		super.setFocus();
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update
	 * (eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void update(CdmDataChangeMap map) {
		if(dataChangeBehavior == null){
			dataChangeBehavior = new KeyEditorDataChangeBehaviour(this);
		}
		DataChangeBridge.handleDataChange(map, dataChangeBehavior);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder
	 * ()
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider#changed(
	 * java.lang.Object)
	 */
	@Override
	public void changed(Object element) {
		editorDirtyStateChanged();
		IEditorPart activeEditor = getActiveEditor();
		if (activeEditor instanceof IDirtyMarkableSelectionProvider) {
			((IDirtyMarkableSelectionProvider) activeEditor).changed(element);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.editor.FormEditor#editorDirtyStateChanged()
	 */
	@Override
	public void editorDirtyStateChanged() {
		dirty = true;
		super.editorDirtyStateChanged();
	}

	@Override
	protected void handlePropertyChange(int propertyId) {
		if (propertyId == PROP_DIRTY) {
			setDirty(true);
		}
		super.handlePropertyChange(propertyId);
	}

	private void setDirty(boolean dirty) {
		this.dirty = dirty;
		firePropertyChange(PROP_DIRTY);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.editor.FormEditor#isDirty()
	 */
	@Override
	public boolean isDirty() {
		return dirty;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.editor.FormEditor#dispose()
	 */
	@Override
	public void dispose() {
		conversation.unregisterForDataStoreChanges(this);
		conversation.close();
		super.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.editor.FormEditor#addPages()
	 */
	@Override
	protected void addPages() {
		AbstractIdentificationEditorInput input = (AbstractIdentificationEditorInput) getEditorInput();
		conversation = input.getConversationHolder();
		conversation.registerForDataStoreChanges(this);

		try {
			addPage(0, new PolytomousKeyListEditor(this), input);
			setPageText(0, "List");
			addPage(1, new PolytomousKeyGraphEditor(this), input);
			setPageText(1, "Graph");
		} catch (PartInitException e) {
			EditorUtil.error(getClass(), e);
		}

		

		setPartName();

	}

	/**
	 * 
	 */
	public void setPartName() {
		AbstractIdentificationEditorInput input = (AbstractIdentificationEditorInput) getEditorInput();
		PolytomousKey key = (PolytomousKey) input.getKey();
		setPartName(key.getTitleCache());
	}
}
