/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>ChangeConceptRelationshipTypeOperation class.</p>
 *
 * @author n.hoffmann
 * @created 03.02.2009
 * @version 1.0
 */
public class ChangeConceptRelationshipTypeOperation extends AbstractPostTaxonOperation {

	private Taxon relatedTaxon;

	private TaxonRelationship taxonRelationship;

	private TaxonRelationshipType oldRelationshipType;

	private TaxonRelationshipType newRelationshipType;

	/**
	 * <p>Constructor for ChangeConceptRelationshipTypeOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param relatedTaxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param type a {@link eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public ChangeConceptRelationshipTypeOperation(String label,
			IUndoContext undoContext, Taxon taxon, Taxon relatedTaxon, TaxonRelationshipType type, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		this.relatedTaxon = relatedTaxon;

		Set<TaxonRelationship> taxonRelationships = taxon.getTaxonRelations(relatedTaxon);

		if(taxonRelationships.size() > 1){
			StoreUtil.warningDialog("Multiple relations between taxa", this, "There are multiple relations between the " +
					"accepted and the related taxon. This case is not handled by the software yet");
			return;
		}

		this.taxonRelationship = taxonRelationships.iterator().next();
		this.oldRelationshipType = taxonRelationship.getType();
		this.newRelationshipType = type;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Remove relatedTaxon and old relationship
		// FIXME since taxon relationships come in a set, which relationships are we going to delete here?
	    element.removeTaxon(relatedTaxon, oldRelationshipType);
		monitor.worked(20);

        // Add new relationship
		// TODO add microcitation for misapplied name to property sheet
		relatedTaxon.addTaxonRelation(element, newRelationshipType, null, null);
		monitor.worked(40);

		return postExecute(relatedTaxon);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// see FIXME in execute()
		StoreUtil.warn(this.getClass(), "Not implemented yet. See developer documentation for details");
		return null;
	}
}
