// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept;

import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.key.AbstractGraphKeyEditor;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;

/**
 * @author n.hoffmann
 * @created Jan 24, 2011
 * @version 1.0
 */
public class ConceptViewPart extends AbstractCdmEditorViewPart implements IPartContentHasDetails{

	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.concept";

	private ListViewer viewer;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if(EditorUtil.getActiveEditor() == null){
			showEmptyPage();
			return;
		}

		if(part instanceof BulkEditor){
			showEmptyPage();
			return;
		}

		if(part instanceof AbstractGraphKeyEditor){
			showEmptyPage();
			return;
		}

		if(part instanceof MultiPageTaxonEditor){
			if(! part.equals(this.part)){
				IEditorInput input = ((IEditorPart) part).getEditorInput();
				showViewer(part, new StructuredSelection(input));
			}
			showViewer();
		}

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#onComplete()
	 */
	@Override
	public boolean onComplete() {
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.AbstractCdmViewPart#getViewer()
	 */
	@Override
	public Viewer getViewer() {
		return viewer;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.AbstractCdmViewPart#createViewer(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createViewer(Composite parent) {
		viewer = new ListViewer(parent);

		viewer.setContentProvider(new ConceptContentProvider());
		viewer.setLabelProvider(new ConceptLabelProvider());


		viewer.setSorter(new ConceptViewerSorter());

		getSite().setSelectionProvider(viewer);

		createMenu();

		createToolbar();
	}

	private void createMenu(){
		MenuManager menuManager = new MenuManager();
		menuManager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));

		getSite().registerContextMenu(menuManager, viewer);

		Control control = viewer.getControl();
		Menu menu = menuManager.createContextMenu(control);

		control.setMenu(menu);
	}

	private void createToolbar() {
		IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();
//		toolBarManager.add();
//		toolBarManager.add();
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.view.AbstractCdmEditorViewPart#changed(java.lang.Object)
	 */
	@Override
	public void changed(Object object) {
		getViewer().refresh();
		super.changed(object);
	}
}
