package eu.etaxonomy.taxeditor.editor.view.derivate;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.cdm.model.occurrence.DerivationEvent;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;

public class DerivateContentProvider implements ITreeContentProvider {

	@Override
	public void dispose() {
		// TODO Auto-generated method stub

	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}

	@Override
	public Object[] getElements(Object inputElement) {
	    SpecimenOrObservationBase specimen = null;
	    if(inputElement instanceof DerivateViewEditorInput){
	        specimen = ((DerivateViewEditorInput) inputElement).getDerivate();
	    }

	    specimen = EditorUtil.getTopMostDerivate(specimen);

	    if(specimen!=null){
	        return Collections.singleton(specimen).toArray();
	    }
	    else {
	        //show topmost element in the derivate hierarchy if no field unit was found
	        return Collections.singleton(specimen).toArray();
	    }
	}

	@Override
	public Object[] getChildren(Object parentElement) {
        if(parentElement instanceof DerivateViewEditorInput){
            parentElement = ((DerivateViewEditorInput) parentElement).getDerivate();
	    }
        if(parentElement instanceof DnaSample){
            DnaSample dnaSample = (DnaSample)parentElement;
            List<Object> children = new ArrayList<Object>();
            //order is important: users want to see Sequences first
            children.addAll(dnaSample.getSequences());
            children.addAll(dnaSample.getAmplifications());
            return children.toArray();
        }
        else if(parentElement instanceof Amplification){
         return ((Amplification) parentElement).getSingleReads().toArray();
        }
        else if(parentElement instanceof Sequence){
            return ((Sequence) parentElement).getSingleReads().toArray();
        }
        else if(parentElement instanceof SpecimenOrObservationBase){
	        return getDerivates((SpecimenOrObservationBase) parentElement).toArray();
	    }
        else if(parentElement instanceof SingleRead){
            List<Object> children = new ArrayList<Object>();
            children.add(((SingleRead) parentElement).getAmplification());
            //TODO getConsensusSequence??
            return children.toArray();
        }
	    return null;
	}

	@Override
	public Object getParent(Object element) {
        if(element instanceof DerivateViewEditorInput){
            element = ((DerivateViewEditorInput) element).getDerivate();
	    }

        if(element instanceof FieldUnit){
            return element;
        }
        else if(element instanceof DerivedUnit){
	        DerivedUnit derivedUnit = (DerivedUnit)element;
	        return derivedUnit.getOriginalUnit();
	    }
        else if(element instanceof Amplification){
            ((Amplification) element).getDnaSample();
        }
        else if(element instanceof Sequence){
            ((Sequence) element).getDnaSample();
        }
        else if(element instanceof SingleRead){
            // TODO there are multiple possible parents
            // does returning null help?
            return null;
        }
		return null;
	}

	@Override
	public boolean hasChildren(Object element) {
        if(element instanceof DerivateViewEditorInput){
            element = ((DerivateViewEditorInput) element).getDerivate();
	    }

        if(element instanceof DnaSample){
            DnaSample dnaSample = (DnaSample)element;
            if(!dnaSample.getSequences().isEmpty() || !dnaSample.getAmplifications().isEmpty()){
                return true;
            }
        }
        else if(element instanceof Amplification){
            Amplification amplification = (Amplification)element;
            return !amplification.getSingleReads().isEmpty();
        }
        else if(element instanceof Sequence){
            Sequence sequence = (Sequence)element;
            return !sequence.getSingleReads().isEmpty();
        }
        else if(element instanceof SpecimenOrObservationBase){
	        SpecimenOrObservationBase specimenOrObservation = (SpecimenOrObservationBase)element;
	        for(DerivationEvent event:(Set<DerivationEvent>)specimenOrObservation.getDerivationEvents()){
	            if(!event.getDerivatives().isEmpty()){
	                return true;
	            }
	        }
	    }
		return false;
	}

	private List<DerivedUnit> getDerivates(SpecimenOrObservationBase specimenOrObservation){
	    List<DerivedUnit> derivates = new ArrayList<DerivedUnit>();
	    for(DerivationEvent derivationEvent:(Set<DerivationEvent>)specimenOrObservation.getDerivationEvents()){
	        derivates.addAll(derivationEvent.getDerivatives());
	    }
	    return derivates;
	}

}
