// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteDescriptionElementOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.media.operation.DeleteMediaOperation;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>DeleteDescriptionHandler class.</p>
 *
 * @author n.hoffmann
 * @created Jun 22, 2010
 * @version 1.0
 */
public class DeleteHandler extends AbstractHandler {
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		IStructuredSelection selection = (IStructuredSelection) HandlerUtil.getActiveSite(event).getSelectionProvider().getSelection();

		IWorkbenchPart part = HandlerUtil.getActivePart(event);
		IPostOperationEnabled postOperationEnabled = (part instanceof IPostOperationEnabled) ? (IPostOperationEnabled) part : null;
		
		try {
			String label = event.getCommand().getName();
		
			IUndoContext undoContext = EditorUtil.getUndoContext();
			
			List<AbstractPostTaxonOperation> operations = new ArrayList<AbstractPostTaxonOperation>();
			
			for(Object object : selection.toArray()){
				
				AbstractPostOperation operation = null;
				
				// TaxonDescription
				if(object instanceof TaxonDescription){
					operations.add(new DeleteTaxonDescriptionOperation(label, undoContext, (TaxonDescription) object, postOperationEnabled));
				}
				// DescriptionElementBase
				else if(object instanceof DescriptionElementBase){
					operations.add(new DeleteDescriptionElementOperation(label, undoContext, (DescriptionElementBase) object, postOperationEnabled));
				}
				else if(object instanceof FeatureNodeContainer){
					List<DescriptionElementBase> descriptions = ((FeatureNodeContainer) object).getDescriptionElementsForEntireBranch();
					
					for(DescriptionElementBase description : descriptions){
						operations.add(new DeleteDescriptionElementOperation(label, undoContext, description, postOperationEnabled)); 
					}
				}
				// Media
				else if(object instanceof Media){
					TreeSelection treeSelection = (TreeSelection) selection;
					
					TreePath[] path = treeSelection.getPathsFor(object);
					
					TaxonDescription imageGallery = (TaxonDescription) path[0].getFirstSegment();
										
					operations.add(new DeleteMediaOperation(label, undoContext, imageGallery, (Media) object, postOperationEnabled));
				}
				else{
					EditorUtil.error(getClass(), "Selection is not valid for this delete handler", null);
				}	
			}
			
			// execute all cumulated operations
			for(AbstractPostOperation operation : operations){
				EditorUtil.executeOperation(operation);
			}
			
		} catch (NotDefinedException e) {
			EditorUtil.warn(getClass(), "Command name not set.");
		}
		
		
		return null;
	}
}
