/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;

/**
 * Property tester used by the description tree menu.
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @version $Id: $
 */
public class DescriptionsMenuPropertyTester extends PropertyTester {

	private static final String MEDIA = "isMedia";
	private static final String FEATURE_NODE_CONTAINER = "isFeatureNodeContainer";
	private static final String DESCRIPTION = "isDescription";
	private static final String DESCRIPTION_ELEMENT = "isDescriptionElement";
	private static final String DELETABLE = "isDeletable";
	private static final String IMAGE_GALLERY = "isImageGallery";

	/* (non-Javadoc)
	 * @see org.eclipse.core.expressions.IPropertyTester#test(java.lang.Object, java.lang.String, java.lang.Object[], java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
    public boolean test(Object receiver, String property, Object[] args,
			Object expectedValue) {

		Object[] selectedElements = ((IStructuredSelection) receiver).toArray();

		if(selectedElements.length == 0){
			// nothing selected so all tests should fail
			return false;
		}

		if(MEDIA.equals(property)){
			return isMedia(selectedElements);
		}
		else if(FEATURE_NODE_CONTAINER.equals(property)){
			return isFeatureNodeContainer(selectedElements);
		}
		else if(DESCRIPTION.equals(property)){
			return isDescription(selectedElements);
		}
		else if(DESCRIPTION_ELEMENT.equals(property)){
			return isDescriptionElement(selectedElements);
		}
		else if(DELETABLE.equals(property)){
			return isDeletable(selectedElements);
		}
		else if(IMAGE_GALLERY.equals(property)){
			return isImageGallery(selectedElements);
		}
		else{
			return false;
		}
	}

	private boolean isImageGallery(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof DescriptionBase) || !((DescriptionBase) object).isImageGallery()){
				return false;
			}
		}
		return true;
	}

	private boolean isFeatureNodeContainer(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof FeatureNodeContainer)){
				return false;
			}
		}
		return true;
	}

	private boolean isDeletable(Object[] selectedElements) {
		boolean result = false;

		for (Object object : selectedElements) {

			if((object instanceof DescriptionBase)
				|| (object instanceof DescriptionElementBase)
				|| (object instanceof Media)
				|| (object instanceof FeatureNodeContainer)){
				result = true;
			}else{
				return false;
			}

		}
		return result;
	}

	private boolean isDescriptionElement(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof DescriptionElementBase)){
				return false;
			}
		}
		return true;
	}

	private boolean isDescription(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof DescriptionBase)){
				return false;
			}
		}
		return true;
	}

	private boolean isMedia(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof Media)){
				return false;
			}
		}
		return true;
	}
}
