// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.media;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.SpecimenDescription;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;

/**
 * <p>MediaContentProvider class.</p>
 *
 * @author n.hoffmann
 * @created Jun 15, 2010
 * @version 1.0
 */
public class MediaContentProvider implements ITreeContentProvider{

	private static final Object[] NO_CHILDREN = new Object[0];
	
	/** {@inheritDoc} */
	public Object[] getChildren(Object parentElement) {
				
		if (parentElement instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) parentElement).getTaxon();
			if(taxon == null){
				EditorUtil.error(getClass(), "Taxon is null", null);
				return NO_CHILDREN;
			}
			HashSet<DescriptionBase> imageGalleries = new HashSet<DescriptionBase>();
			for(DescriptionBase description : taxon.getDescriptions()){
				if(description.isImageGallery()){
					imageGalleries.add(description);
				}
			}			
			return imageGalleries.toArray(); 
		}
		else if (parentElement instanceof DescriptionBase) {
			if (((DescriptionBase) parentElement).isImageGallery()) {
				return getImages((DescriptionBase) parentElement).toArray();
			} 
		}
		else if (parentElement instanceof DerivedUnit){
			try {
				DerivedUnitFacade facade = DerivedUnitFacade.NewInstance((DerivedUnit) parentElement);
		
				// TODO at the moment we always create image galleries because the facade is like so.
				// this should definitely change and we want to use normal getters and setters here
				List<DescriptionBase> derivedUnitFacadeImageGalleries = new ArrayList<DescriptionBase>(2);
				
				SpecimenDescription derivedUnitImageGallery = facade.getDerivedUnitImageGallery(false);
				SpecimenDescription fieldObjectImageGallery = facade.getFieldObjectImageGallery(false);
				
				if(derivedUnitImageGallery != null){
					derivedUnitFacadeImageGalleries.add(derivedUnitImageGallery);
				}
				
				if(fieldObjectImageGallery != null){
					derivedUnitFacadeImageGalleries.add(fieldObjectImageGallery);
				}
				
				return derivedUnitFacadeImageGalleries.toArray();
				
			} catch (DerivedUnitFacadeNotSupportedException e) {
				EditorUtil.error(this.getClass(), "DerivedUnitFacadeNotSupportedException when trying to instantiate DerivedUnitFacade", e);
			}
		}
		
		return NO_CHILDREN;
	}

	/** {@inheritDoc} */
	public Object getParent(Object element) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	public boolean hasChildren(Object element) {
		return (getChildren(element).length > 0);
	}

	/** {@inheritDoc} */
	public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}

	/**
	 * <p>dispose</p>
	 */
	public void dispose() {}

	/** {@inheritDoc} */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
	
	private List<Media> getImages(DescriptionBase description){
		Assert.isTrue(description.isImageGallery(), "Description should have the imageGallery flag set.");
		
		Set<DescriptionElementBase> elements = description.getElements();
		if (elements != null) {
			// by definition, image galleries have only one description element
			if(elements.size() > 1){
				EditorUtil.error(this.getClass(), "There should be one and only one description element to hold the images. Found: " + elements.size() + " InDescription ID: " + description.getUuid(), null);
			}
						
			DescriptionElementBase element = elements.iterator().next();
			
			return element.getMedia();
		}
		return null;
	}
	
	private SpecimenDescription createDerivedUnitFacadeImageGallery(DerivedUnitFacade facade){
		SpecimenDescription description = SpecimenDescription.NewInstance();
		description.setImageGallery(true);
		
		
		
		return description;
	}
}
